@@ -316,7 +316,7 @@ header_get_epsg = function(header)
 {
   pos <- where_is_epsg(header)
 
-  if (pos == 0)
+  if (pos == 0L)
     return(0)
   else
     return(header[["Variable Length Records"]][["GeoKeyDirectoryTag"]][["tags"]][[pos]][["value offset"]])
@@ -338,7 +338,7 @@ header_set_epsg = function(header, epsg)
       header[["Variable Length Records"]][["GeoKeyDirectoryTag"]][["length after header"]] <- 40
       header[["Variable Length Records"]][["GeoKeyDirectoryTag"]][["description"]]         <- "Geo Key Directory Tag"
       header[["Variable Length Records"]][["GeoKeyDirectoryTag"]][["tags"]]                <- vector("list", 1)
-      pos <- 1
+      pos <- 1L
     }
     else
     {
@@ -356,28 +356,43 @@ header_get_wktcs = function(header)
 {
   wkt = header[["Variable Length Records"]][["WKT OGC CS"]][["WKT OGC COORDINATE SYSTEM"]]
 
-  if (is.null(wkt))
-    return("")
-  else
+  if (!is.null(wkt))
     return(wkt)
+
+  wkt = header[["Extended Variable Length Records"]][["WKT OGC CS"]][["WKT OGC COORDINATE SYSTEM"]]
+
+  if (!is.null(wkt))
+    return(wkt)
+
+  return("")
 }
 
 #' @export
 #' @rdname crs_tools
 header_set_wktcs = function(header, WKT)
 {
-  if (is.null(header[["Variable Length Records"]]))
-    header[["Variable Length Records"]] <- list()
+  vlrs <- length(header[["Variable Length Records"]]) > 0L
+  evlrs <- length(header[["Extended Variable Length Records"]]) > 0L
+  attr <- "Variable Length Records"
 
-  if (is.null(header[["Variable Length Records"]][["WKT OGC CS"]]))
-    header[["Variable Length Records"]][["WKT OGC CS"]] <- list()
+  if (!vlrs && evlrs)
+    attr <- "Extended Variable Length Records"
+
+  if (is.null(header[[attr]]))
+    header[[attr]] <- list()
+
+  if (is.null(header[[attr]][["WKT OGC CS"]]))
+  {
+    attr <- "Variable Length Records"
+    header[[attr]][["WKT OGC CS"]] <- list()
+  }
 
 
-  header[["Variable Length Records"]][["WKT OGC CS"]][["reserved"]]                  <- 43707L
-  header[["Variable Length Records"]][["WKT OGC CS"]][["user ID"]]                   <-  "LASF_Projection"
-  header[["Variable Length Records"]][["WKT OGC CS"]][["record ID"]]                 <- 2112
-  header[["Variable Length Records"]][["WKT OGC CS"]][["description"]]               <- "WKT Information"
-  header[["Variable Length Records"]][["WKT OGC CS"]][["WKT OGC COORDINATE SYSTEM"]] <- WKT
+  header[[attr]][["WKT OGC CS"]][["reserved"]]                  <- 43707L
+  header[[attr]][["WKT OGC CS"]][["user ID"]]                   <-  "LASF_Projection"
+  header[[attr]][["WKT OGC CS"]][["record ID"]]                 <- 2112
+  header[[attr]][["WKT OGC CS"]][["description"]]               <- "WKT Information"
+  header[[attr]][["WKT OGC CS"]][["WKT OGC COORDINATE SYSTEM"]] <- WKT
   header[["Global Encoding"]][["WKT"]] <- TRUE
   return(header)
 }
@@ -392,7 +407,7 @@ where_is_epsg = function(header)
       return(i)
   }
 
-  return(0)
+  return(0L)
 }
 
 guess_las_format <- function(data)
